@@ -2,11 +2,11 @@ import imp
 import logging
 import os
 import sys
-from optparse import make_option
+from optparse import OptionParser, make_option
 
-from django.core.exceptions import ImproperlyConfigured
-from django.core.management import LaxOptionParser
 from django.conf import ENVIRONMENT_VARIABLE as SETTINGS_ENVIRONMENT_VARIABLE
+from django.core.exceptions import ImproperlyConfigured
+from django.core.management import base
 
 from .utils import uppercase_attributes, reraise
 from .values import Value, setup_value
@@ -14,30 +14,44 @@ from .values import Value, setup_value
 installed = False
 
 CONFIGURATION_ENVIRONMENT_VARIABLE = 'DJANGO_CONFIGURATION'
+CONFIGURATION_ARGUMENT = '--configuration'
+CONFIGURATION_ARGUMENT_HELP = ('The name of the configuration class to load, '
+                               'e.g. "Development". If this isn\'t provided, '
+                               'the  DJANGO_CONFIGURATION environment '
+                               'variable will be used.')
 
 
-configuration_options = (
-    make_option('--configuration',
-                help='The name of the configuration class to load, e.g. '
-                     '"Development". If this isn\'t provided, the '
-                     'DJANGO_CONFIGURATION environment variable will '
-                     'be used.'),)
+configuration_options = (make_option(CONFIGURATION_ARGUMENT,
+                                     help=CONFIGURATION_ARGUMENT_HELP),)
 
 
 def install(check_options=False):
     global installed
     if not installed:
-        from django.core.management import base
+        orig_create_parser = base.BaseCommand.create_parser
 
-        # add the configuration option to all management commands
-        base.BaseCommand.option_list += configuration_options
+        def create_parser(self, prog_name, subcommand):
+            parser = orig_create_parser(self, prog_name, subcommand)
+            if isinstance(parser, OptionParser):
+                # in case the option_list is set the create_parser
+                # will actually return a OptionParser for backward
+                # compatibility. In that case we should tack our
+                # options on to the end of the parser on the way out.
+                for option in configuration_options:
+                    parser.add_option(option)
+            else:
+                # probably argparse, let's not import argparse though
+                parser.add_argument(CONFIGURATION_ARGUMENT,
+                                    help=CONFIGURATION_ARGUMENT_HELP)
+            return parser
 
+        base.BaseCommand.create_parser = create_parser
         importer = ConfigurationImporter(check_options=check_options)
         sys.meta_path.insert(0, importer)
         installed = True
 
 
-class ConfigurationImporter(object):
+class ConfigurationImporter:
     modvar = SETTINGS_ENVIRONMENT_VARIABLE
     namevar = CONFIGURATION_ENVIRONMENT_VARIABLE
     error_msg = ("Configuration cannot be imported, "
@@ -68,13 +82,22 @@ class ConfigurationImporter(object):
         return os.environ.get(self.namevar)
 
     def check_options(self):
-        parser = LaxOptionParser(option_list=configuration_options,
-                                 add_help_option=False)
+        parser = base.CommandParser(
+            usage="%(prog)s subcommand [options] [args]",
+            add_help=False,
+        )
+        parser.add_argument('--settings')
+        parser.add_argument('--pythonpath')
+        parser.add_argument(CONFIGURATION_ARGUMENT,
+                            help=CONFIGURATION_ARGUMENT_HELP)
+
+        parser.add_argument('args', nargs='*')  # catch-all
         try:
-            options, args = parser.parse_args(self.argv)
+            options, args = parser.parse_known_args(self.argv[2:])
             if options.configuration:
                 os.environ[self.namevar] = options.configuration
-        except:
+            base.handle_default_options(options)
+        except base.CommandError:
             pass  # Ignore any option errors at this point.
 
     def validate(self):
@@ -87,23 +110,20 @@ class ConfigurationImporter(object):
         if len(self.argv) > 1:
             from . import __version__
             from django.utils.termcolors import colorize
-            # Django >= 1.7 supports hiding the colorization in the shell
-            try:
-                from django.core.management.color import no_style
-            except ImportError:
-                no_style = None
+            from django.core.management.color import no_style
 
-            if no_style is not None and '--no-color' in self.argv:
+            if '--no-color' in self.argv:
                 stylize = no_style()
             else:
-                stylize = lambda text: colorize(text, fg='green')
+                def stylize(text):
+                    return colorize(text, fg='green')
 
-            if (self.argv[1] == 'runserver' and
-                    os.environ.get('RUN_MAIN') == 'true'):
+            if (self.argv[1] == 'runserver'
+                    and os.environ.get('RUN_MAIN') == 'true'):
 
                 message = ("django-configurations version {0}, using "
-                           "configuration '{1}'".format(__version__,
-                                                        self.name))
+                           "configuration {1}".format(__version__ or "",
+                                                      self.name))
                 self.logger.debug(stylize(message))
 
     def find_module(self, fullname, path=None):
@@ -114,7 +134,7 @@ class ConfigurationImporter(object):
         return None
 
 
-class ConfigurationLoader(object):
+class ConfigurationLoader:
 
     def __init__(self, name, location):
         self.name = name
