@@ -6,30 +6,61 @@ import sys
 
 from django.core import validators
 from django.core.exceptions import ValidationError, ImproperlyConfigured
-from django.utils import six
+from django.utils.module_loading import import_string
 
-from .utils import import_by_path
+from .utils import getargspec
 
 
 def setup_value(target, name, value):
     actual_value = value.setup(name)
     # overwriting the original Value class with the result
-    setattr(target, name, actual_value)
+    setattr(target, name, value.value)
     if value.multiple:
         for multiple_name, multiple_value in actual_value.items():
             setattr(target, multiple_name, multiple_value)
 
 
-class Value(object):
+class Value:
     """
     A single settings value that is able to interpret env variables
     and implements a simple validation scheme.
     """
     multiple = False
+    late_binding = False
+    environ_required = False
+
+    @property
+    def value(self):
+        value = self.default
+        if not hasattr(self, '_value') and self.environ_name:
+            self.setup(self.environ_name)
+        if hasattr(self, '_value'):
+            value = self._value
+        return value
+
+    @value.setter
+    def value(self, value):
+        self._value = value
+
+    def __new__(cls, *args, **kwargs):
+        """
+        checks if the creation can end up directly in the final value.
+        That is the case whenever environ = False or environ_name is given.
+        """
+        instance = object.__new__(cls)
+        if 'late_binding' in kwargs:
+            instance.late_binding = kwargs.get('late_binding')
+        if not instance.late_binding:
+            instance.__init__(*args, **kwargs)
+            if ((instance.environ and instance.environ_name)
+                    or (not instance.environ and instance.default)):
+                instance = instance.setup(instance.environ_name)
+        return instance
 
     def __init__(self, default=None, environ=True, environ_name=None,
-                 environ_prefix='DJANGO', *args, **kwargs):
-        if isinstance(default, Value):
+                 environ_prefix='DJANGO', environ_required=False,
+                 *args, **kwargs):
+        if isinstance(default, Value) and default.default is not None:
             self.default = copy.copy(default.default)
         else:
             self.default = default
@@ -38,24 +69,43 @@ class Value(object):
             environ_prefix = environ_prefix[:-1]
         self.environ_prefix = environ_prefix
         self.environ_name = environ_name
+        self.environ_required = environ_required
+
+    def __str__(self):
+        return str(self.value)
 
     def __repr__(self):
-        return "<Value default: {0}>".format(self.default)
+        return repr(self.value)
+
+    def __eq__(self, other):
+        return self.value == other
+
+    def __bool__(self):
+        return bool(self.value)
+
+    # Compatibility with python 2
+    __nonzero__ = __bool__
+
+    def full_environ_name(self, name):
+        if self.environ_name:
+            environ_name = self.environ_name
+        else:
+            environ_name = name.upper()
+        if self.environ_prefix:
+            environ_name = '{0}_{1}'.format(self.environ_prefix, environ_name)
+        return environ_name
 
     def setup(self, name):
         value = self.default
         if self.environ:
-            if self.environ_name is None:
-                environ_name = name.upper()
-            else:
-                environ_name = self.environ_name
-            if self.environ_prefix:
-                full_environ_name = '{0}_{1}'.format(self.environ_prefix,
-                                                     environ_name)
-            else:
-                full_environ_name = environ_name
+            full_environ_name = self.full_environ_name(name)
             if full_environ_name in os.environ:
                 value = self.to_python(os.environ[full_environ_name])
+            elif self.environ_required:
+                raise ValueError('Value {0!r} is required to be set as the '
+                                 'environment variable {1!r}'
+                                 .format(name, full_environ_name))
+        self.value = value
         return value
 
     def to_python(self, value):
@@ -67,7 +117,7 @@ class Value(object):
         return value
 
 
-class MultipleMixin(object):
+class MultipleMixin:
     multiple = True
 
 
@@ -76,7 +126,7 @@ class BooleanValue(Value):
     false_values = ('no', 'n', 'false', '0', '')
 
     def __init__(self, *args, **kwargs):
-        super(BooleanValue, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.default not in (True, False):
             raise ValueError('Default value {0!r} is not a '
                              'boolean value'.format(self.default))
@@ -92,24 +142,36 @@ class BooleanValue(Value):
                              'boolean value {0!r}'.format(value))
 
 
-class CastingMixin(object):
+class CastingMixin:
     exception = (TypeError, ValueError)
     message = 'Cannot interpret value {0!r}'
 
     def __init__(self, *args, **kwargs):
-        super(CastingMixin, self).__init__(*args, **kwargs)
-        if isinstance(self.caster, six.string_types):
-            self._caster = import_by_path(self.caster)
+        super().__init__(*args, **kwargs)
+        if isinstance(self.caster, str):
+            try:
+                self._caster = import_string(self.caster)
+            except ImportError as err:
+                msg = "Could not import {!r}".format(self.caster)
+                raise ImproperlyConfigured(msg) from err
         elif callable(self.caster):
             self._caster = self.caster
         else:
             error = 'Cannot use caster of {0} ({1!r})'.format(self,
                                                               self.caster)
             raise ValueError(error)
+        try:
+            arg_names = getargspec(self._caster)[0]
+            self._params = {name: kwargs[name] for name in arg_names if name in kwargs}
+        except TypeError:
+            self._params = {}
 
     def to_python(self, value):
         try:
-            return self._caster(value)
+            if self._params:
+                return self._caster(value, **self._params)
+            else:
+                return self._caster(value)
         except self.exception:
             raise ValueError(self.message.format(value))
 
@@ -118,6 +180,15 @@ class IntegerValue(CastingMixin, Value):
     caster = int
 
 
+class PositiveIntegerValue(IntegerValue):
+
+    def to_python(self, value):
+        int_value = super().to_python(value)
+        if int_value < 0:
+            raise ValueError(self.message.format(value))
+        return int_value
+
+
 class FloatValue(CastingMixin, Value):
     caster = float
 
@@ -127,89 +198,134 @@ class DecimalValue(CastingMixin, Value):
     exception = decimal.InvalidOperation
 
 
-class ListValue(Value):
+class SequenceValue(Value):
+    """
+    Common code for sequence-type values (lists and tuples).
+    Do not use this class directly. Instead use a subclass.
+    """
+
+    # Specify this value in subclasses, e.g. with 'list' or 'tuple'
+    sequence_type = None
     converter = None
-    message = 'Cannot interpret list item {0!r} in list {1!r}'
 
     def __init__(self, *args, **kwargs):
+        msg = 'Cannot interpret {0} item {{0!r}} in {0} {{1!r}}'
+        self.message = msg.format(self.sequence_type.__name__)
         self.separator = kwargs.pop('separator', ',')
         converter = kwargs.pop('converter', None)
         if converter is not None:
             self.converter = converter
-        super(ListValue, self).__init__(*args, **kwargs)
-        # make sure the default is a list
+        super().__init__(*args, **kwargs)
+        # make sure the default is the correct sequence type
         if self.default is None:
-            self.default = []
+            self.default = self.sequence_type()
+        else:
+            self.default = self.sequence_type(self.default)
         # initial conversion
         if self.converter is not None:
-            self.default = [self.converter(value) for value in self.default]
+            self.default = self._convert(self.default)
+
+    def _convert(self, sequence):
+        converted_values = []
+        for value in sequence:
+            try:
+                converted_values.append(self.converter(value))
+            except (TypeError, ValueError):
+                raise ValueError(self.message.format(value, value))
+        return self.sequence_type(converted_values)
 
     def to_python(self, value):
         split_value = [v.strip() for v in value.strip().split(self.separator)]
         # removing empty items
-        value_list = filter(None, split_value)
-        if self.converter is None:
-            return list(value_list)
+        value_list = self.sequence_type(filter(None, split_value))
+        if self.converter is not None:
+            value_list = self._convert(value_list)
+        return self.sequence_type(value_list)
 
-        converted_values = []
-        for list_value in value_list:
-            try:
-                converted_values.append(self.converter(list_value))
-            except (TypeError, ValueError):
-                raise ValueError(self.message.format(list_value, value))
-        return converted_values
+
+class ListValue(SequenceValue):
+    sequence_type = list
+
+
+class TupleValue(SequenceValue):
+    sequence_type = tuple
+
+
+class SingleNestedSequenceValue(SequenceValue):
+    """
+    Common code for nested sequences (list of lists, or tuple of tuples).
+    Do not use this class directly. Instead use a subclass.
+    """
+
+    def __init__(self, *args, **kwargs):
+        self.seq_separator = kwargs.pop('seq_separator', ';')
+        super().__init__(*args, **kwargs)
+
+    def _convert(self, items):
+        # This could receive either a bare or nested sequence
+        if items and isinstance(items[0], self.sequence_type):
+            converted_sequences = [
+                super(SingleNestedSequenceValue, self)._convert(i) for i in items
+            ]
+            return self.sequence_type(converted_sequences)
+        return self.sequence_type(super()._convert(items))
+
+    def to_python(self, value):
+        split_value = [
+            v.strip() for v in value.strip().split(self.seq_separator)
+        ]
+        # Remove empty items
+        filtered = self.sequence_type(filter(None, split_value))
+        sequence = [
+            super(SingleNestedSequenceValue, self).to_python(f) for f in filtered
+        ]
+        return self.sequence_type(sequence)
+
+
+class SingleNestedListValue(SingleNestedSequenceValue):
+    sequence_type = list
+
+
+class SingleNestedTupleValue(SingleNestedSequenceValue):
+    sequence_type = tuple
 
 
 class BackendsValue(ListValue):
 
     def converter(self, value):
         try:
-            import_by_path(value)
-        except ImproperlyConfigured as err:
-            six.reraise(ValueError, ValueError(err), sys.exc_info()[2])
+            import_string(value)
+        except ImportError as err:
+            raise ValueError(err).with_traceback(sys.exc_info()[2])
         return value
 
 
-class TupleValue(ListValue):
-    message = 'Cannot interpret tuple item {0!r} in tuple {1!r}'
-
-    def __init__(self, *args, **kwargs):
-        super(TupleValue, self).__init__(*args, **kwargs)
-        if self.default is None:
-            self.default = ()
-        else:
-            self.default = tuple(self.default)
-
-    def to_python(self, value):
-        return tuple(super(TupleValue, self).to_python(value))
-
-
 class SetValue(ListValue):
     message = 'Cannot interpret set item {0!r} in set {1!r}'
 
     def __init__(self, *args, **kwargs):
-        super(SetValue, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.default is None:
             self.default = set()
         else:
             self.default = set(self.default)
 
     def to_python(self, value):
-        return set(super(SetValue, self).to_python(value))
+        return set(super().to_python(value))
 
 
 class DictValue(Value):
     message = 'Cannot interpret dict value {0!r}'
 
     def __init__(self, *args, **kwargs):
-        super(DictValue, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.default is None:
             self.default = {}
         else:
             self.default = dict(self.default)
 
     def to_python(self, value):
-        value = super(DictValue, self).to_python(value)
+        value = super().to_python(value)
         if not value:
             return {}
         try:
@@ -221,18 +337,23 @@ class DictValue(Value):
         return evaled_value
 
 
-class ValidationMixin(object):
+class ValidationMixin:
 
     def __init__(self, *args, **kwargs):
-        super(ValidationMixin, self).__init__(*args, **kwargs)
-        if isinstance(self.validator, six.string_types):
-            self._validator = import_by_path(self.validator)
+        super().__init__(*args, **kwargs)
+        if isinstance(self.validator, str):
+            try:
+                self._validator = import_string(self.validator)
+            except ImportError as err:
+                msg = "Could not import {!r}".format(self.validator)
+                raise ImproperlyConfigured(msg) from err
         elif callable(self.validator):
             self._validator = self.validator
         else:
             raise ValueError('Cannot use validator of '
                              '{0} ({1!r})'.format(self, self.validator))
-        self.to_python(self.default)
+        if self.default:
+            self.to_python(self.default)
 
     def to_python(self, value):
         try:
@@ -264,19 +385,19 @@ class RegexValue(ValidationMixin, Value):
     def __init__(self, *args, **kwargs):
         regex = kwargs.pop('regex', None)
         self.validator = validators.RegexValidator(regex=regex)
-        super(RegexValue, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
 
 class PathValue(Value):
     def __init__(self, *args, **kwargs):
         self.check_exists = kwargs.pop('check_exists', True)
-        super(PathValue, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def setup(self, name):
-        value = super(PathValue, self).setup(name)
+        value = super().setup(name)
         value = os.path.expanduser(value)
         if self.check_exists and not os.path.exists(value):
-            raise ValueError('Path {0!r} does  not exist.'.format(value))
+            raise ValueError('Path {0!r} does not exist.'.format(value))
         return os.path.abspath(value)
 
 
@@ -284,13 +405,14 @@ class SecretValue(Value):
 
     def __init__(self, *args, **kwargs):
         kwargs['environ'] = True
-        super(SecretValue, self).__init__(*args, **kwargs)
+        kwargs['environ_required'] = True
+        super().__init__(*args, **kwargs)
         if self.default is not None:
             raise ValueError('Secret values are only allowed to '
                              'be set as environment variables')
 
     def setup(self, name):
-        value = super(SecretValue, self).setup(name)
+        value = super().setup(name)
         if not value:
             raise ValueError('Secret value {0!r} is not set'.format(name))
         return value
@@ -299,12 +421,13 @@ class SecretValue(Value):
 class EmailURLValue(CastingMixin, MultipleMixin, Value):
     caster = 'dj_email_url.parse'
     message = 'Cannot interpret email URL value {0!r}'
+    late_binding = True
 
     def __init__(self, *args, **kwargs):
         kwargs.setdefault('environ', True)
         kwargs.setdefault('environ_prefix', None)
         kwargs.setdefault('environ_name', 'EMAIL_URL')
-        super(EmailURLValue, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.default is None:
             self.default = {}
         else:
@@ -319,14 +442,14 @@ class DictBackendMixin(Value):
         kwargs.setdefault('environ', True)
         kwargs.setdefault('environ_prefix', None)
         kwargs.setdefault('environ_name', self.environ_name)
-        super(DictBackendMixin, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         if self.default is None:
             self.default = {}
         else:
             self.default = self.to_python(self.default)
 
     def to_python(self, value):
-        value = super(DictBackendMixin, self).to_python(value)
+        value = super().to_python(value)
         return {self.alias: value}
 
 
@@ -334,15 +457,18 @@ class DatabaseURLValue(DictBackendMixin, CastingMixin, Value):
     caster = 'dj_database_url.parse'
     message = 'Cannot interpret database URL value {0!r}'
     environ_name = 'DATABASE_URL'
+    late_binding = True
 
 
 class CacheURLValue(DictBackendMixin, CastingMixin, Value):
     caster = 'django_cache_url.parse'
     message = 'Cannot interpret cache URL value {0!r}'
     environ_name = 'CACHE_URL'
+    late_binding = True
 
 
 class SearchURLValue(DictBackendMixin, CastingMixin, Value):
     caster = 'dj_search_url.parse'
     message = 'Cannot interpret Search URL value {0!r}'
     environ_name = 'SEARCH_URL'
+    late_binding = True
