@@ -2,6 +2,7 @@ import decimal
 import os
 from contextlib import contextmanager
 
+from django import VERSION as DJANGO_VERSION
 from django.test import TestCase
 from django.core.exceptions import ImproperlyConfigured
 
@@ -411,6 +412,7 @@ class ValueTests(TestCase):
                 'EMAIL_HOST_PASSWORD': 'password',
                 'EMAIL_HOST_USER': 'user@domain.com',
                 'EMAIL_PORT': 587,
+                'EMAIL_TIMEOUT': None,
                 'EMAIL_USE_SSL': False,
                 'EMAIL_USE_TLS': True})
         with env(EMAIL_URL='console://'):
@@ -421,6 +423,7 @@ class ValueTests(TestCase):
                 'EMAIL_HOST_PASSWORD': None,
                 'EMAIL_HOST_USER': None,
                 'EMAIL_PORT': None,
+                'EMAIL_TIMEOUT': None,
                 'EMAIL_USE_SSL': False,
                 'EMAIL_USE_TLS': False})
         with env(EMAIL_URL='smtps://user@domain.com:password@smtp.example.com:wrong'):  # noqa: E501
@@ -429,7 +432,7 @@ class ValueTests(TestCase):
     def test_cache_url_value(self):
         cache_setting = {
             'default': {
-                'BACKEND': 'django_redis.cache.RedisCache',
+                'BACKEND': 'django_redis.cache.RedisCache' if DJANGO_VERSION < (4,) else 'django.core.cache.backends.redis.RedisCache',  # noqa: E501
                 'LOCATION': 'redis://host:6379/1',
             }
         }
@@ -503,6 +506,7 @@ class ValueTests(TestCase):
                 'EMAIL_HOST_PASSWORD': 'password',
                 'EMAIL_HOST_USER': 'user@domain.com',
                 'EMAIL_PORT': 587,
+                'EMAIL_TIMEOUT': None,
                 'EMAIL_USE_SSL': False,
                 'EMAIL_USE_TLS': True
             })
